@@ -9,7 +9,7 @@ import logging
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
-#logger.setLevel(logging.DEBUG)
+# logger.setLevel(logging.DEBUG)
 logger.setLevel(logging.INFO)
 
 ATTR_BRIGHTNESS = "brightness"
@@ -41,6 +41,45 @@ DOBISS_TEMPERATURE = 204
 DOBISS_AUDIO = 205
 DOBISS_FLAG = 206
 
+ICON_FROM_DOBISS = {
+    DOBISS_LIGHT: "mdi:lightbulb",
+    DOBISS_PLUG: "mdi:power-plug",
+    DOBISS_VENTILATION: "mdi:hvac",
+    DOBISS_UP: "mdi:arrow-up",
+    DOBISS_DOWN: "mdi:arrow-down",
+    DOBISS_HEATING: "mdi:radiator",
+    DOBISS_TABLELIGHT: "mdi:lamp",
+    DOBISS_DOOR: "mdi:door",
+    DOBISS_GARAGE: "mdi:garage",
+    DOBISS_GATE: "mdi:gate",
+    DOBISS_RED: "mdi:exclamation",
+    DOBISS_GREEN: "mdi:thumb-up",
+    DOBISS_BLUE: "mdi:help",
+    DOBISS_WHITE: "mdi:alpha-n",
+    DOBISS_INPUTSTATUS: "mdi:list-status",
+    DOBISS_LIGHTSENSOR: "mdi:theme-light-dark",
+    DOBISS_SCENARIO: "mdi:movie-open",
+    DOBISS_AUTOMATION: "mdi:home-automation",
+    DOBISS_CONDITION: "mdi:account-question",
+    DOBISS_TEMPERATURE: "mdi:thermometer",
+    DOBISS_AUDIO: "mdi:cast-audio",
+    DOBISS_FLAG: "mdi:flag",
+}
+
+# DOBISS type mapping
+DOBISS_TYPE_NXT = 0
+DOBISS_TYPE_INPUT = 1
+DOBISS_TYPE_DALI = 1
+DOBISS_TYPE_RELAIS = 8
+DOBISS_TYPE_ANALOG = 24
+DOBISS_TYPE_SCENARIO = 201
+DOBISS_TYPE_AUTOMATION = 202
+DOBISS_TYPE_CONDITION = 203
+DOBISS_TYPE_TEMPERATURE = 204
+DOBISS_TYPE_AUDIO = 205
+DOBISS_TYPE_FLAG = 206
+
+
 class DobissEntity:
     """ a generic Dobiss Entity, can be a light, switch, sensor, etc... """
 
@@ -134,14 +173,14 @@ class DobissEntity:
         """Schedule call all registered callbacks."""
         for callback in self._callbacks:
             callback()
-            
+
     async def push(self, val):
         """when an external status udate happened, and you want to update the internal value"""
         if self._value != val:
             logger.debug(f"Updated {self._name} to {val}")
             self._value = val
             await self.publish_updates()
-    
+
     async def update_from_global(self, status):
         """when an external status udate happened, and you want to update the internal value and parse the update data here to fetch what is needed"""
         try:
@@ -149,15 +188,22 @@ class DobissEntity:
                 line = status[str(self.address)]
                 if type(line) == list and len(status[str(self.address)]) > self.channel:
                     await self.push(status[str(self.address)][self.channel])
-                elif type(line) == dict and str(self.channel) in status[str(self.address)]:
+                elif (
+                    type(line) == dict
+                    and str(self.channel) in status[str(self.address)]
+                ):
                     if self.address == DOBISS_TEMPERATURE:
-                        await self.push(float(status[str(self.address)][str(self.channel)]['temp']))
+                        await self.push(
+                            float(status[str(self.address)][str(self.channel)]["temp"])
+                        )
                     else:
-                        await self.push(int(status[str(self.address)][str(self.channel)]))
-                #else:
+                        await self.push(
+                            int(status[str(self.address)][str(self.channel)])
+                        )
+                # else:
                 #    logger.debug(f"{self.name} not found in status update")
-                #logger.debug("Updated {} = {}: groupname {}; addr {}; channel {}; dimmable {}".format(self.name, self.value, self.groupname, self.address, self.channel, self.dimmable))
-            #else:
+                # logger.debug("Updated {} = {}: groupname {}; addr {}; channel {}; dimmable {}".format(self.name, self._value, self.groupname, self.address, self.channel, self.dimmable))
+            # else:
             #    logger.debug("{} not found in status update".format(self.name))
         except Exception:
             logger.exception("Error trying to update {}".format(self.name))
@@ -169,12 +215,13 @@ class DobissEntity:
         response = await self._dobiss.status(self._address, self._channel)
         data = await response.json()
         if self._address == DOBISS_TEMPERATURE:
-            val = float(data["status"]['temp'])
+            val = float(data["status"]["temp"])
         else:
             val = int(data["status"])
         if val != self._value:
-	        self._value  = val
-	        await self.publish_updates()
+            self._value = val
+            await self.publish_updates()
+
 
 class DobissOutput(DobissEntity):
     """ a generic Dobiss Output, can be a light, switch, etc... """
@@ -200,50 +247,64 @@ class DobissOutput(DobissEntity):
         """Instruct the entity to turn off."""
         await self._dobiss.action(self._address, self._channel, 0)
 
+
 class DobissLight(DobissOutput):
     """ a dobiss light object, can be dimmable or not """
 
+
+class DobissAnalogOutput(DobissOutput):
+    """ a dobiss light object, can be dimmable or not """
+
+
 class DobissSwitch(DobissOutput):
     """ a dobiss switch, can be up/down switch, door switch, etc... """
 
+
 class DobissScenario(DobissSwitch):
     """ a dobiss scenario """
 
+
 class DobissAutomation(DobissSwitch):
     """ a dobiss automation """
 
+
 class DobissFlag(DobissSwitch):
     """ a dobiss flag """
 
+
 class DobissSensor(DobissEntity):
     """ a dobiss sensor, can be binary or not, lightswitch, temperature sensor, etc """
+
     def __init__(self, dobiss, data, groupname):
-        DobissEntity.__init__(self, dobiss, data, groupname)
+        super().__init__(dobiss, data, groupname)
         if int(data["type"]) == DOBISS_TEMPERATURE:
-            self._unit = 'C'
+            self._unit = "C"
         elif int(data["type"]) == 0:
-            self._unit = '%'
+            self._unit = "%"
         else:
             self._unit = None
 
     @property
     def unit(self):
         return self._unit
-    
+
     def set_unit(self, unit):
         self._unit = unit
 
+
 class DobissTempSensor(DobissSensor):
     """ a dobiss Temperature Sensor """
 
+
 class DobissBinarySensor(DobissSensor):
     """ a dobiss Binary Sensor """
 
+
 class DobissLightSensor(DobissSensor):
     """ a dobiss Light Sensor """
 
+
 class DobissAPI:
-    
     def __init__(self, secret, host, secure: bool):
         """ Initialize dobiss api object """
         url = ""
@@ -257,7 +318,7 @@ class DobissAPI:
 
         self._host = host
         self._secure = secure
-        self._token = ''
+        self._token = ""
         self._exp_time = datetime.now()
         self._secret = secret
         self._url = url
@@ -288,11 +349,13 @@ class DobissAPI:
         return self._session
 
     async def auth_check(self):
-        headers = { 'Authorization': 'Bearer ' + self.get_token() }
+        headers = {"Authorization": "Bearer " + self.get_token()}
         auth_ok = False
         try:
             self.start_session()
-            async with self._session.get(self._url + 'status', headers=headers) as response:
+            async with self._session.get(
+                self._url + "status", headers=headers
+            ) as response:
                 if response and response.status == 200:
                     auth_ok = True
         except:
@@ -305,7 +368,13 @@ class DobissAPI:
         """ Request a token to use in a request to the Dobiss server """
         if self._exp_time < datetime.now() + timedelta(hours=20):
             # get new token
-            self._token = (jwt.encode({'name': 'my_application'}, self._secret, headers={'expiresIn': "24h" })).decode("utf-8")
+            self._token = (
+                jwt.encode(
+                    {"name": "my_application"},
+                    self._secret,
+                    headers={"expiresIn": "24h"},
+                )
+            ).decode("utf-8")
             self._exp_time = datetime.now() + timedelta(hours=20)
         return self._token
 
@@ -321,7 +390,7 @@ class DobissAPI:
                 f"Discovery interval below {MIN_DISCOVERY_INTERVAL} seconds is invalid"
             )
         self._discovery_interval = val
-        
+
     def _call_discovery(self):
         if not self._last_discovery or self._force_discovery:
             self._force_discovery = False
@@ -336,9 +405,11 @@ class DobissAPI:
     async def discovery(self):
         if self._call_discovery():
             try:
-                headers = { 'Authorization': 'Bearer ' + self.get_token() }
+                headers = {"Authorization": "Bearer " + self.get_token()}
                 self.start_session()
-                response = await self._session.get(self._url + 'discover', headers=headers)
+                response = await self._session.get(
+                    self._url + "discover", headers=headers
+                )
                 if response and response.status == 200:
                     discovered_devices = await response.json()
                     logger.debug(f"Discover response: {discovered_devices}")
@@ -349,49 +420,90 @@ class DobissAPI:
             logger.debug("Discovery: Use cached info")
         return self._devices
 
-    async def status(self, address = None, channel = None):
+    async def status(self, address=None, channel=None):
         data = {}
         if address != None:
             data["address"] = address
         if channel != None:
             data["channel"] = channel
-        headers = { 'Authorization': 'Bearer ' + self.get_token() }
+        headers = {"Authorization": "Bearer " + self.get_token()}
         self.start_session()
-        return await self._session.get(self._url + 'status', headers=headers, json=data)
+        return await self._session.get(self._url + "status", headers=headers, json=data)
 
-    async def action(self, address, channel, action, option1 = None):
-        writedata = { 'address': address, 'channel': channel, 'action': action }
+    async def action(self, address, channel, action, option1=None):
+        writedata = {"address": address, "channel": channel, "action": action}
         if option1 != None:
             writedata["option1"] = option1
-        headers = { 'Authorization': 'Bearer ' + self.get_token() }
+        headers = {"Authorization": "Bearer " + self.get_token()}
         self.start_session()
-        return await self._session.post(self._url + 'action', headers=headers, json=writedata)
+        return await self._session.post(
+            self._url + "action", headers=headers, json=writedata
+        )
 
     def _get_dobiss_devices(self, discovered_devices):
         new_devices = []
         for group in discovered_devices["groups"]:
             for subject in group["subjects"]:
-                logger.debug("Discovered {}: addr {}; channel {}; type {}; icon {}".format(subject["name"], subject["address"], subject["channel"], subject["type"], subject["icons_id"]))
+                logger.debug(
+                    "Discovered {}: addr {}; channel {}; type {}; icon {}".format(
+                        subject["name"],
+                        subject["address"],
+                        subject["channel"],
+                        subject["type"],
+                        subject["icons_id"],
+                    )
+                )
                 if group["group"]["id"] != 0:
                     # skip first group - nothing of interest in there...
-                    if str(subject["icons_id"]) == "0": # check for lights
-                        new_devices.append(DobissLight(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "8": # other items connected to a relais
-                        new_devices.append(DobissSwitch(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "1": # status input
-                        new_devices.append(DobissBinarySensor(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "206": # flags
-                        new_devices.append(DobissFlag(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "201": # scenarios
-                        new_devices.append(DobissScenario(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "202": # automations
-                        new_devices.append(DobissAutomation(self, subject, group["group"]["name"]))
-                    #elif str(subject["type"]) == "203": # logical conditions
-                    #	new_devices.append(DobissSensor(self, subject, group["group"]["name"]))
-                    elif (str(subject["type"]) == "204" and subject["name"] != "All zones"): # temperature
-                        new_devices.append(DobissTempSensor(self, subject, group["group"]["name"]))
-                    elif str(subject["type"]) == "0": # lightcell
-                        new_devices.append(DobissLightSensor(self, subject, group["group"]["name"]))
+                    if str(subject["icons_id"]) == str(
+                        DOBISS_LIGHT
+                    ):  # check for lights
+                        new_devices.append(
+                            DobissLight(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(
+                        DOBISS_TYPE_ANALOG
+                    ):  # other items connected to a 0-10V output
+                        new_devices.append(
+                            DobissAnalogOutput(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(
+                        DOBISS_TYPE_RELAIS
+                    ):  # other items connected to a relais
+                        new_devices.append(
+                            DobissSwitch(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(DOBISS_TYPE_INPUT):  # status input
+                        new_devices.append(
+                            DobissBinarySensor(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(DOBISS_TYPE_FLAG):  # flags
+                        new_devices.append(
+                            DobissFlag(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(DOBISS_TYPE_SCENARIO):  # scenarios
+                        new_devices.append(
+                            DobissScenario(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(
+                        DOBISS_TYPE_AUTOMATION
+                    ):  # automations
+                        new_devices.append(
+                            DobissAutomation(self, subject, group["group"]["name"])
+                        )
+                    # elif str(subject["type"]) == "203": # logical conditions
+                    # 	new_devices.append(DobissSensor(self, subject, group["group"]["name"]))
+                    elif (
+                        str(subject["type"]) == str(DOBISS_TYPE_AUDIO)
+                        and subject["name"] != "All zones"
+                    ):  # temperature
+                        new_devices.append(
+                            DobissTempSensor(self, subject, group["group"]["name"])
+                        )
+                    elif str(subject["type"]) == str(DOBISS_TYPE_NXT):  # lightcell
+                        new_devices.append(
+                            DobissLightSensor(self, subject, group["group"]["name"])
+                        )
         for dev in new_devices:
             existing_dev = self.get_device_by_id(dev.object_id)
             if existing_dev:
@@ -400,11 +512,11 @@ class DobissAPI:
                 # a new device - add this to the list
                 self._devices.append(dev)
         return self._devices
-    
+
     def get_devices_by_type(self, dev_type):
         device_list = []
         for device in self._devices:
-            if type(device) == dev_type:
+            if isinstance(device, dev_type):
                 device_list.append(device)
         return device_list
 
@@ -430,39 +542,32 @@ class DobissAPI:
     async def listen_for_dobiss(self):
         while not self._stop_monitoring:
             logger.debug("registering for websocket connection")
-            headers = { 'Authorization': 'Bearer ' + self.get_token() }
+            headers = {"Authorization": "Bearer " + self.get_token()}
             self.start_session()
-            self._websocket = await self._session.ws_connect(self._ws_url, headers=headers)
+            ws = await self._session.ws_connect(self._ws_url, headers=headers)
             while not self._stop_monitoring:
                 try:
-                    response = await self._websocket.receive()
+                    response = await ws.receive_json()
                     # logger.debug("received ws message")
-                    if response.type == aiohttp.WSMsgType.CLOSE:
-                        await self._websocket.close()
-                        break
-                    elif response.type == aiohttp.WSMsgType.ERROR:
-                        break
-                    elif response.type == aiohttp.WSMsgType.CLOSED:
-                        break
-                    if (
-                        response.type == aiohttp.WSMsgType.TEXT
-                        and response.data[0] == "{"
-                    ):
-                        logger.debug(f"Status update pushed: {response.data}")
-                        await self.update_from_status(response.json())
+                    logger.debug(f"Status update pushed: {response}")
+                    await self.update_from_status(response)
+                except TypeError:
+                    logger.exception("dobiss monitor exception")
+                except ValueError:
+                    logger.exception("dobiss monitor exception")
                 except asyncio.exceptions.CancelledError:
-                    logger.debug("websocket connection cancelled - stop monitoring")
+                    logger.debug("websocket connection cancelled - we must be stopping")
                     self._stop_monitoring = True
+                    break
                 except:
                     logger.exception(f"Status update exception")
+                    if not ws.closed:
+                        await ws.close()
                     break
 
-    async def stop_monitoring(self):
+    def stop_monitoring(self):
         self._stop_monitoring = True
-        await self._websocket.close()
 
     async def dobiss_monitor(self):
         self._stop_monitoring = False
         asyncio.ensure_future(self.listen_for_dobiss())
-
-
